@@ -27,6 +27,7 @@
 #ifndef _CHCANV_H__
 #define _CHCANV_H__
 
+#include <cstdint>
 #include "bbox.h"
 
 #include <wx/datetime.h>
@@ -36,6 +37,7 @@
 #include <wx/grid.h>
 #include <wx/wxhtml.h>
 
+#include "model/nmea_log.h"
 #include "ocpndc.h"
 #include "undo.h"
 
@@ -44,6 +46,8 @@
 #include "emboss_data.h"
 #include "S57Sector.h"
 #include "gshhs.h"
+#include "notification_manager_gui.h"
+#include "observable.h"
 
 class wxGLContext;
 class GSHHSChart;
@@ -126,14 +130,29 @@ enum {
 
 enum { NORTH_UP_MODE, COURSE_UP_MODE, HEAD_UP_MODE };
 
-//----------------------------------------------------------------------------
-// ChartCanvas
-//----------------------------------------------------------------------------
+/**
+ * ChartCanvas - Main chart display and interaction component
+ *
+ * Manages the visualization of nautical charts and all chart-related user
+ * interactions. This class integrates geographic data, user interface elements,
+ * and navigation tools into a comprehensive chart viewing experience. It
+ * handles everything from chart loading and rendering to user gestures, route
+ * planning, and navigation aids.
+ *
+ * ChartCanvas uses ViewPort internally to handle the mathematical
+ * transformations between geographic and screen coordinates. While ViewPort
+ * focuses on the projection math, ChartCanvas manages the actual rendering,
+ * user interaction, and application logic.
+ *
+ * As a wxWindow subclass, it responds to paint, mouse, keyboard, and other
+ * window events, translating them into appropriate chart operations like
+ * panning, zooming, and object manipulation.
+ */
 class ChartCanvas : public wxWindow {
   friend class glChartCanvas;
 
 public:
-  ChartCanvas(wxFrame *frame, int canvasIndex);
+  ChartCanvas(wxFrame *frame, int canvasIndex, wxWindow *nmea_log);
   ~ChartCanvas();
 
   void SetupGlCanvas();
@@ -145,6 +164,8 @@ public:
   void OnPaint(wxPaintEvent &event);
   void PaintCleanup();
   void Scroll(int dx, int dy);
+  void ResetGridFont() { m_pgridFont = nullptr; }
+
   void OnToolLeftClick(wxCommandEvent &event);
 
   bool MouseEventOverlayWindows(wxMouseEvent &event);
@@ -155,6 +176,21 @@ public:
 
   bool MouseEventSetup(wxMouseEvent &event, bool b_handle_dclick = true);
   bool MouseEventProcessObjects(wxMouseEvent &event);
+  /**
+   * Processes mouse events for core chart panning and zooming operations.
+   *
+   * This method handles the main chart navigation interactions:
+   * - Mouse wheel zooming with configurable sensitivity and optional smoothing.
+   * - Left-click chart centering. Centers chart on click point in non-drag
+   * scenarios.
+   * - Click-and-drag panning. Updates chart position continuously during drag.
+   *
+   * @param event The mouse event to process
+   * @return true if event was processed, false to allow focus shifting between
+   * canvases.
+   *
+   * @note Interacts with route creation and measurement tools active states
+   */
   bool MouseEventProcessCanvas(wxMouseEvent &event);
   void SetCanvasCursor(wxMouseEvent &event);
   void OnKillFocus(wxFocusEvent &WXUNUSED(event));
@@ -193,14 +229,52 @@ public:
   void LostMouseCapture(wxMouseCaptureLostEvent &event);
 
   void CancelMouseRoute();
+  /**
+   * Set the width of the screen in millimeters.
+   */
   void SetDisplaySizeMM(double size);
+  /**
+   * Get the width of the screen in millimeters.
+   */
   double GetDisplaySizeMM() { return m_display_size_mm; }
 
+  /**
+   * Sets the viewport scale while maintaining the center point.
+   *
+   * Changes the chart display scale while preserving the current view center,
+   * orientation, skew, and projection settings. This is typically used for zoom
+   * operations.
+   *
+   * @param scale The new viewport scale to set (pixels per meter)
+   * @param refresh Whether to refresh the display after changing the scale
+   * @return bool True if the scale change was successful
+   */
   bool SetVPScale(double sc, bool b_refresh = true);
   bool SetVPProjection(int projection);
+  /**
+   * Set the viewport center point.
+   * Centers the view on a specific lat/lon position.
+   * @param lat Latitude in degrees
+   * @param lon Longitude in degrees
+   * @return true if view was changed successfully
+   */
   bool SetViewPoint(double lat, double lon);
   bool SetViewPointByCorners(double latSW, double lonSW, double latNE,
                              double lonNE);
+  /**
+   * Set the viewport center point, scale, skew, rotation and projection.
+   *
+   * @param lat Latitude of viewport center in degrees.
+   * @param lon Longitude of viewport center in degrees.
+   * @param scale_ppm Requested viewport scale in physical pixels per meter.
+   * This is the desired rendering scale before projection effects.
+   * @param skew Chart skew angle in radians.
+   * @param rotation Viewport rotation in radians.
+   * @param projection Projection type (default=0). If 0, maintains current
+   * projection.
+   * @param b_adjust Allow small viewport adjustments for performance.
+   * @param b_refresh Request screen refresh after change.
+   */
   bool SetViewPoint(double lat, double lon, double scale_ppm, double skew,
                     double rotation, int projection = 0, bool b_adjust = true,
                     bool b_refresh = true);
@@ -229,18 +303,99 @@ public:
   double GetVPRotation(void) { return GetVP().rotation; }
   double GetVPSkew(void) { return GetVP().skew; }
   double GetVPTilt(void) { return GetVP().tilt; }
+
   void ClearbFollow(void);
   void SetbFollow(void);
   void TogglebFollow(void);
+  bool GetbFollow() { return m_bFollow; }
+
   void JumpToPosition(double lat, double lon, double scale);
   void SetFirstAuto(bool b_auto) { m_bFirstAuto = b_auto; }
 
+  /**
+   * Convert latitude/longitude to canvas pixel coordinates (physical pixels)
+   * with double precision.
+   *
+   * Returns unrounded floating point pixel coordinates. When used with drawing
+   * functions that take integer coordinates, values will be truncated.
+   *
+   * @param rlat Latitude in degrees
+   * @param rlon Longitude in degrees
+   * @param r [out] Pointer to wxPoint2DDouble to receive the canvas pixel
+   * coordinates as unrounded floating point values
+   */
   void GetDoubleCanvasPointPix(double rlat, double rlon, wxPoint2DDouble *r);
+  /**
+   * Convert latitude/longitude to canvas pixel coordinates (physical pixels)
+   * with double precision, using specified viewport.
+   *
+   * Returns unrounded floating point pixel coordinates. When used with drawing
+   * functions that take integer coordinates, values will be truncated.
+   *
+   * @param vp ViewPort containing projection parameters and canvas settings
+   * @param rlat Latitude in degrees
+   * @param rlon Longitude in degrees
+   * @param r [out] Pointer to wxPoint2DDouble to receive the canvas pixel
+   * coordinates as unrounded floating point values
+   */
   void GetDoubleCanvasPointPixVP(ViewPort &vp, double rlat, double rlon,
                                  wxPoint2DDouble *r);
+
+  /**
+   * Convert latitude/longitude to canvas pixel coordinates (physical pixels)
+   * rounded to nearest integer.
+   *
+   * Uses GetDoubleCanvasPointPixVP internally and rounds results using wxRound
+   * (std::lround). This means 3.7 becomes 4, -3.7 becomes -4.
+   *
+   * @param rlat Latitude in degrees
+   * @param rlon Longitude in degrees
+   * @param r [out] Pointer to wxPoint to receive the canvas pixel coordinates
+   * in physical pixels, as rounded integer values.
+   * @return true if conversion successful, false if coordinates are invalid or
+   * out of bounds.
+   *
+   * @see ViewPort::GetPixFromLL() for the underlying coordinate transformation.
+   */
   bool GetCanvasPointPix(double rlat, double rlon, wxPoint *r);
+
+  /**
+   * Convert latitude/longitude to canvas pixel coordinates rounded to nearest
+   * integer using specified viewport.
+   *
+   * Uses GetDoubleCanvasPointPixVP internally and rounds results using wxRound
+   * (std::lround). This means 3.7 becomes 4, -3.7 becomes -4.
+   *
+   * @param vp ViewPort containing projection parameters and canvas settings
+   * @param rlat Latitude in degrees
+   * @param rlon Longitude in degrees
+   * @param r [out] Pointer to wxPoint to receive the canvas pixel coordinates
+   * in physical pixels, as rounded integer values.
+   * @return true if conversion successful, false if:
+   *         - Coordinates would be on other side of the world (resulting in
+   * NaN)
+   *         - Resulting pixel values would be too large (>1e6)
+   *         In these cases, r is set to INVALID_COORD
+   *
+   * @see ViewPort::GetPixFromLL() for the underlying coordinate transformation
+   */
   bool GetCanvasPointPixVP(ViewPort &vp, double rlat, double rlon, wxPoint *r);
 
+  /**
+   * Convert canvas pixel coordinates (physical pixels) to latitude/longitude.
+   *
+   * Uses BSB chart geo-referencing for compatible raster charts when conditions
+   * permit, otherwise falls back to viewport projection estimation.
+   *
+   * @param x X-coordinate in physical pixels
+   * @param y Y-coordinate in physical pixels
+   * @param lat [out] Reference to receive the resulting latitude at the given
+   * (x, y) coordinates.
+   * @param lon [out] Reference to receive the resulting longitude at the given
+   * (x, y) coordinates.
+   *
+   * @see ViewPort::GetLLFromPix() for the underlying coordinate transformation.
+   */
   void GetCanvasPixPoint(double x, double y, double &lat, double &lon);
   void WarpPointerDeferred(int x, int y);
   void UpdateShips();
@@ -250,6 +405,10 @@ public:
 
   bool IsMeasureActive() { return m_bMeasure_Active; }
   wxBitmap &GetTideBitmap() { return m_cTideBitmap; }
+  Undo *undo;
+
+  int GetUpMode() { return m_upMode; }
+  bool GetLookahead() { return m_bLookAhead; }
 
   void UnlockQuilt();
   void SetQuiltMode(bool b_quilt);
@@ -269,9 +428,29 @@ public:
 
   bool StartTimedMovement(bool stoptimer = true);
   void DoTimedMovement();
+  /**
+   * Performs a step of smooth movement animation on the chart canvas.
+   *
+   * This function is called to update the canvas position, scale, or rotation
+   * during animated panning, zooming, or rotating operations. The amount of
+   * movement performed is determined by the elapsed time parameter @p dt, and
+   * the operation continues as long as the internal movement timer is nonzero.
+   *
+   * @param dt Elapsed time in milliseconds since the last movement step.
+   *
+   * @see StartTimedMovement(), StopMovement(), m_mustmove
+   */
   void DoMovement(long dt);
   void StopMovement();
 
+  void StartTimedMovementVP(double target_lat, double target_lon, int nstep);
+  void DoTimedMovementVP();
+  void StopMovementVP();
+
+  void StartTimedMovementTarget();
+  void DoTimedMovementTarget();
+  void StopMovementTarget();
+
   void SetColorScheme(ColorScheme cs);
   ColorScheme GetColorScheme() { return m_cs; }
 
@@ -281,12 +460,25 @@ public:
   //    Accessors
   int GetCanvasWidth() { return m_canvas_width; }
   int GetCanvasHeight() { return m_canvas_height; }
+  /** Return the ViewPort scale factor, in physical pixels per meter. */
   float GetVPScale() { return GetVP().view_scale_ppm; }
+  /** Return the ViewPort chart scale denominator (e.g., 50000 for a 1:50000
+   * scale). */
   float GetVPChartScale() { return GetVP().chart_scale; }
+  /**
+   * Return the number of logical pixels per meter for the screen.
+   *
+   * @todo The name of this function is misleading. It should be renamed to
+   * GetCanvasLogicalPixelsPerMeter() or similar. It looks like some callers
+   * are expecting the physical pixels per meter, which is incorrect.
+   */
   double GetCanvasScaleFactor() { return m_canvas_scale_factor; }
+  /**
+   * Return the physical pixels per meter at chart center, accounting for
+   * latitude distortion.
+   */
   double GetCanvasTrueScale() { return m_true_scale_ppm; }
   double GetAbsoluteMinScalePpm() { return m_absolute_min_scale_ppm; }
-  ViewPort &GetVP();
   ViewPort *GetpVP() { return &VPoint; }
   void SetVP(ViewPort &);
   ChartBase *GetChartAtCursor();
@@ -310,20 +502,50 @@ public:
   bool GetbShowTide() { return m_bShowTide; }
   void SetShowVisibleSectors(bool val) { m_bShowVisibleSectors = val; }
   bool GetShowVisibleSectors() { return m_bShowVisibleSectors; }
-
+  /** Get the number of logical pixels per millimeter on the screen. */
   double GetPixPerMM() { return m_pix_per_mm; }
 
   void SetOwnShipState(ownship_state_t state) { m_ownship_state = state; }
   void SetCursorStatus(double cursor_lat, double cursor_lon);
   void GetCursorLatLon(double *lat, double *lon);
-
+  /** Pans (moves) the canvas by the specified physical pixels in x and y
+   * directions. */
   bool PanCanvas(double dx, double dy);
   void StopAutoPan(void);
 
+  /**
+   * Perform a smooth zoom operation on the chart canvas by the specified
+   * factor.
+   *
+   * The zoom can either be centered on the cursor position or the center of the
+   * viewport.
+   *
+   * @param factor The zoom factor to apply:
+   *              - factor > 1: Zoom in, e.g. 2.0 makes objects twice as large
+   *              - factor < 1: Zoom out, e.g. 0.5 makes objects half as large
+   * @param can_zoom_to_cursor If true, zoom operation will be centered on
+   * current cursor position. If false, zoom operation centers on current
+   * viewport center.
+   * @param stoptimer If true, stops any ongoing movement/zoom operations before
+   * starting this zoom. If false, allows this zoom to blend with existing
+   * operations.
+   */
+
   void ZoomCanvas(double factor, bool can_zoom_to_cursor = true,
                   bool stoptimer = true);
+
+  /**
+   * Perform an immediate zoom operation without smooth transitions.
+   *
+   * This is a simplified version of ZoomCanvas that applies the zoom
+   * immediately without animation or cursor-centering logic. Typically used for
+   * programmatic zooming.
+   *
+   * @param factor The zoom factor to apply:
+   *              - factor > 1: Zoom in, e.g. 2.0 makes objects twice as large
+   *              - factor < 1: Zoom out, e.g. 0.5 makes objects half as large
+   */
   void ZoomCanvasSimple(double factor);
-  void DoZoomCanvas(double factor, bool can_zoom_to_cursor = true);
 
   void RotateCanvas(double dir);
   void DoRotateCanvas(double rotation);
@@ -335,6 +557,80 @@ public:
   void HideGlobalToolbar();
   void ShowGlobalToolbar();
 
+  bool GetShowDepthUnits() { return m_bShowDepthUnits; }
+  void SetShowDepthUnits(bool show) { m_bShowDepthUnits = show; }
+  bool GetShowGrid() { return m_bDisplayGrid; }
+  void SetShowGrid(bool show) { m_bDisplayGrid = show; }
+  bool GetShowOutlines() { return m_bShowOutlines; }
+  void SetShowOutlines(bool show) { m_bShowOutlines = show; }
+  bool GetShowChartbar() { return true; }
+  wxRect GetMUIBarRect();
+  void SetMUIBarPosition();
+  void DestroyMuiBar();
+  void CreateMUIBar();
+
+  void ToggleChartOutlines(void);
+  void ToggleCanvasQuiltMode(void);
+
+  wxString GetScaleText() { return m_scaleText; }
+  double GetScaleValue() { return m_scaleValue; }
+  bool GetShowAIS() { return m_bShowAIS; }
+  void SetShowAIS(bool show);
+  bool GetAttenAIS() { return m_bShowAISScaled; }
+  void SetAttenAIS(bool show);
+  void SetShowFocusBar(bool enable) { m_show_focus_bar = enable; }
+  bool GetShowFocusBar() { return m_show_focus_bar; }
+  MUIBar *GetMUIBar() { return m_muiBar; }
+  void SetAlertString(wxString str) { m_alertString = str; }
+  wxString GetAlertString() { return m_alertString; }
+  bool GetShowENCText() { return m_encShowText; }
+  void SetShowENCText(bool show);
+
+  bool GetShowENCDepth() { return m_encShowDepth; }
+  void SetShowENCDepth(bool show);
+
+  bool GetShowENCLightDesc() { return m_encShowLightDesc; }
+  void SetShowENCLightDesc(bool show);
+
+  bool GetShowENCBuoyLabels() { return m_encShowBuoyLabels; }
+  void SetShowENCBuoyLabels(bool show);
+
+  bool GetShowENCLights() { return m_encShowLights; }
+  void SetShowENCLights(bool show);
+
+  int GetENCDisplayCategory() { return m_encDisplayCategory; }
+  void SetENCDisplayCategory(int category);
+
+  bool GetShowENCAnchor() { return m_encShowAnchor; }
+  void SetShowENCAnchor(bool show);
+
+  bool GetShowENCDataQual() { return m_encShowDataQual; }
+  void SetShowENCDataQual(bool show);
+
+  void JaggyCircle(ocpnDC &dc, wxPen pen, int x, int y, int radius);
+  int m_canvasIndex;
+  void ShowTides(bool bShow);
+  void ShowCurrents(bool bShow);
+  void SetUpMode(int mode);
+  void ToggleLookahead();
+  void SetShowGPS(bool show);
+  void UpdateFollowButtonState(void);
+  void InvalidateGL();
+  bool IsTileOverlayIndexInYesShow(int index);
+  bool IsTileOverlayIndexInNoShow(int index);
+  void AddTileOverlayIndexToNoShow(int index);
+  int m_groupIndex;
+  Route *m_pMouseRoute;
+  bool m_bMeasure_Active;
+  ViewPort &GetVP();
+  ChartBase *m_singleChart;
+  Quilt *m_pQuilt;
+  wxString FindValidUploadPort();
+  wxString m_active_upload_port;
+
+  // protected:
+
+  // private:
   ChartBase *GetLargestScaleQuiltChart();
   ChartBase *GetFirstQuiltChart();
   ChartBase *GetNextQuiltChart();
@@ -354,9 +650,6 @@ public:
   int GetCanvasChartNativeScale();
   int FindClosestCanvasChartdbIndex(int scale);
   void UpdateCanvasOnGroupChange(void);
-  void SetUpMode(int mode);
-  void ToggleLookahead();
-  void SetShowGPS(bool show);
 
   void ShowObjectQueryWindow(int x, int y, float zlat, float zlon);
   void ShowMarkPropertiesDialog(RoutePoint *markPoint);
@@ -401,12 +694,13 @@ public:
 
   bool IsPianoContextMenuActive() { return m_piano_ctx_menu != 0; }
   bool DoCanvasCOGSet(void);
-  void UpdateFollowButtonState(void);
   void ApplyGlobalSettings();
   void SetShowGPSCompassWindow(bool bshow);
-
+  bool GetShowGPSCompassWindow() { return m_bShowCompassWin; }
   void FreezePiano() { m_pianoFrozen = true; }
   void ThawPiano() { m_pianoFrozen = false; }
+  void StartChartDragInertia();
+  void SetupGridFont();
 
   // Todo build more accessors
   bool m_bFollow;
@@ -417,21 +711,46 @@ public:
   TCWin *pCwin;
   wxBitmap *pscratch_bm;
   bool m_brepaint_piano;
-  double m_cursor_lon, m_cursor_lat;
-  Undo *undo;
+  /**
+   * The longitude in degrees corresponding to the most recently processed
+   * cursor position.
+   *
+   * This variable does NOT continuously track the mouse cursor position. It is
+   * updated only:
+   * 1. When processing left mouse clicks
+   * 2. During panning operations via mouse dragging
+   * 3. When the OnCursorTrackTimerEvent fires after mouse movement stops
+   * 4. During certain object selection and editing operations
+   *
+   * For code that needs the current geographic coordinates under the current
+   * mouse pointer, use GetCanvasPixPoint(mouse_x, mouse_y, lat, lon) instead of
+   * accessing this variable directly.
+   */
+  double m_cursor_lon;
+  /**
+   * The latitude in degrees corresponding to the most recently processed cursor
+   * position.
+   *
+   * This variable does NOT continuously track the mouse cursor position. It is
+   * updated only:
+   * 1. When processing left mouse clicks
+   * 2. During panning operations via mouse dragging
+   * 3. When the OnCursorTrackTimerEvent fires after mouse movement stops
+   * 4. During certain object selection and editing operations
+   *
+   * For code that needs the current geographic coordinates under the current
+   * mouse pointer, use GetCanvasPixPoint(mouse_x, mouse_y, lat, lon) instead of
+   * accessing this variable directly.
+   */
+  double m_cursor_lat;
   wxPoint r_rband;
   double m_prev_rlat;
   double m_prev_rlon;
   RoutePoint *m_prev_pMousePoint;
-  Quilt *m_pQuilt;
   bool m_bShowNavobjects;
-  int m_canvasIndex;
-  int m_groupIndex;
   int m_routeState;
-  ChartBase *m_singleChart;
   int m_upMode;
   bool m_bLookAhead;
-  double m_VPRotate;
 
 #ifdef HAVE_WX_GESTURE_EVENTS
   double m_oldVPSScale;
@@ -445,26 +764,18 @@ public:
   void StartRoute(void);
   void FinishRoute(void);
 
-  void InvalidateGL();
-
 #ifdef ocpnUSE_GL
   glChartCanvas *GetglCanvas() { return m_glcc; }
 #endif
 
-  void JaggyCircle(ocpnDC &dc, wxPen pen, int x, int y, int radius);
-
   bool CheckEdgePan(int x, int y, bool bdragging, int margin, int delta);
 
-  Route *m_pMouseRoute;
   bool m_FinishRouteOnKillFocus;
-  bool m_bMeasure_Active;
   bool m_bMeasure_DistCircle;
-  wxString m_active_upload_port;
   bool m_bAppendingRoute;
   int m_nMeasureState;
   Route *m_pMeasureRoute;
   MyFrame *parent_frame;
-  wxString FindValidUploadPort();
   CanvasMenuHandler *m_canvasMenu;
   int GetMinAvailableGshhgQuality() {
     return pWorldBackgroundChart->GetMinAvailableQuality();
@@ -485,70 +796,12 @@ public:
                             ChartFamilyEnum New_Family = CHART_FAMILY_DONTCARE);
   void SelectdbChart(int dbindex);
 
-  void ShowTides(bool bShow);
-  void ShowCurrents(bool bShow);
-
   void DoCanvasStackDelta(int direction);
 
   void ProcessNewGUIScale();
 
-  bool GetShowDepthUnits() { return m_bShowDepthUnits; }
-  void SetShowDepthUnits(bool show) { m_bShowDepthUnits = show; }
-  bool GetShowGrid() { return m_bDisplayGrid; }
-  void SetShowGrid(bool show) { m_bDisplayGrid = show; }
-  bool GetShowOutlines() { return m_bShowOutlines; }
-  void SetShowOutlines(bool show) { m_bShowOutlines = show; }
-  bool GetShowChartbar() { return true; }
-  wxRect GetMUIBarRect();
-  void SetMUIBarPosition();
-  void DestroyMuiBar();
-  void CreateMUIBar();
-
-  void ToggleChartOutlines(void);
-  void ToggleCanvasQuiltMode(void);
-
-  wxString GetScaleText() { return m_scaleText; }
-  double GetScaleValue() { return m_scaleValue; }
-
   bool m_b_paint_enable;
 
-  bool GetShowENCText() { return m_encShowText; }
-  void SetShowENCText(bool show);
-
-  bool GetShowENCDepth() { return m_encShowDepth; }
-  void SetShowENCDepth(bool show);
-
-  bool GetShowENCLightDesc() { return m_encShowLightDesc; }
-  void SetShowENCLightDesc(bool show);
-
-  bool GetShowENCBuoyLabels() { return m_encShowBuoyLabels; }
-  void SetShowENCBuoyLabels(bool show);
-
-  bool GetShowENCLights() { return m_encShowLights; }
-  void SetShowENCLights(bool show);
-
-  int GetENCDisplayCategory() { return m_encDisplayCategory; }
-  void SetENCDisplayCategory(int category);
-
-  bool GetShowENCAnchor() { return m_encShowAnchor; }
-  void SetShowENCAnchor(bool show);
-
-  bool GetShowENCDataQual() { return m_encShowDataQual; }
-  void SetShowENCDataQual(bool show);
-
-  int GetUpMode() { return m_upMode; }
-  bool GetLookahead() { return m_bLookAhead; }
-
-  bool GetShowAIS() { return m_bShowAIS; }
-  void SetShowAIS(bool show);
-  bool GetAttenAIS() { return m_bShowAISScaled; }
-  void SetAttenAIS(bool show);
-
-  MUIBar *GetMUIBar() { return m_muiBar; }
-
-  void SetAlertString(wxString str) { m_alertString = str; }
-  wxString GetAlertString() { return m_alertString; }
-
   wxRect GetScaleBarRect() { return m_scaleBarRect; }
   void RenderAlertMessage(wxDC &dc, const ViewPort &vp);
 
@@ -556,16 +809,38 @@ public:
   std::vector<int> m_tile_yesshow_index_array;
   std::vector<int> m_quilt_noshow_index_array;
 
-  bool IsTileOverlayIndexInYesShow(int index);
-  bool IsTileOverlayIndexInNoShow(int index);
-  void AddTileOverlayIndexToNoShow(int index);
-
   std::vector<int> GetQuiltNoshowIindexArray() {
     return m_quilt_noshow_index_array;
   }
+  /**
+   * Get the ratio of physical to logical pixel for the display.
+   *
+   * On standard displays, one logical pixel equals one physical pixel, so this
+   * value is 1.0. On high-DPI/Retina displays, one logical pixel may equal
+   * multiple physical pixels:
+   * - MacBook Pro Retina: 2.0 (2x2 physical pixels per logical pixel)
+   * - Other HiDPI displays: May be 1.5, 1.75, etc.
+   */
   double GetDisplayScale() { return m_displayScale; }
+  void ResetOwnshipOffset() { m_OSoffsetx = m_OSoffsety = 0; }
+  NotificationsList *GetNotificationsList() { return m_NotificationsList; }
 
 private:
+  /**
+   * Internal function that implements the actual zoom operation.
+   *
+   * This function handles the core zoom functionality including scale
+   * calculations, chart selection and viewport updates.
+   *
+   * @param factor The zoom factor to apply:
+   *              - factor > 1: Zoom in, e.g. 2.0 makes objects twice as large
+   *              - factor < 1: Zoom out, e.g. 0.5 makes objects half as large
+   *
+   * @param can_zoom_to_cursor If true, zoom operation will be centered on
+   * cursor position. If false, zoom operation centers on viewport center.
+   */
+  void DoZoomCanvas(double factor, bool can_zoom_to_cursor = true);
+
   int AdjustQuiltRefChart();
 
   bool UpdateS52State();
@@ -577,18 +852,20 @@ private:
 
   ViewPort VPoint;
   void PositionConsole(void);
+  wxWindow *m_nmea_log;
 
   wxColour PredColor();
   wxColour ShipColor();
 
   void ComputeShipScaleFactor(float icon_hdt, int ownShipWidth,
-                              int ownShipLength, wxPoint &lShipMidPoint,
-                              wxPoint &GpsOffsetPixels, wxPoint lGPSPoint,
-                              float &scale_factor_x, float &scale_factor_y);
+                              int ownShipLength, wxPoint2DDouble &lShipMidPoint,
+                              wxPoint &GpsOffsetPixels,
+                              wxPoint2DDouble lGPSPoint, float &scale_factor_x,
+                              float &scale_factor_y);
 
-  void ShipDrawLargeScale(ocpnDC &dc, wxPoint lShipMidPoint);
+  void ShipDrawLargeScale(ocpnDC &dc, wxPoint2DDouble lShipMidPoint);
   void ShipIndicatorsDraw(ocpnDC &dc, int img_height, wxPoint GPSOffsetPixels,
-                          wxPoint lGPSPoint);
+                          wxPoint2DDouble lGPSPoint);
 
   ChInfoWin *m_pCIWin;
 
@@ -639,14 +916,15 @@ private:
   bool bShowingCurrent;
   bool bShowingTide;
 
-  double
-      m_canvas_scale_factor;  // converter....
-                              // useage....
-                              // true_chart_scale_on_display =
-                              // m_canvas_scale_factor / pixels_per_meter of
-                              // displayed chart also may be considered as the
-                              // "pixels-per-meter" of the canvas on-screen
-  double m_pix_per_mm;        // pixels per millimeter on the screen
+  /**
+   * Display-specific scale factor in logical pixels per meter for the physical
+   * screen.
+   *
+   * @note This is the same as m_pix_per_mm / 1000.0
+   */
+  double m_canvas_scale_factor;
+  /** Number of logical pixels per millimeter on the screen. */
+  double m_pix_per_mm;
   double m_display_size_mm;
 
   double m_absolute_min_scale_ppm;
@@ -675,6 +953,8 @@ private:
   void MovementStopTimerEvent(wxTimerEvent &);
   void OnCursorTrackTimerEvent(wxTimerEvent &event);
 
+  void MovementVPTimerEvent(wxTimerEvent &event);
+
   void DrawAllTracksInBBox(ocpnDC &dc, LLBBox &BltBBox);
   void DrawActiveTrackInBBox(ocpnDC &dc, LLBBox &BltBBox);
   void DrawAllRoutesInBBox(ocpnDC &dc, LLBBox &BltBBox);
@@ -717,9 +997,13 @@ private:
   void DrawEmboss(ocpnDC &dc, emboss_data *pemboss);
 
   void ShowBrightnessLevelTimedPopup(int brightness, int min, int max);
+  void HandleNotificationMouseClick();
 
   //    Data
-  int m_canvas_width, m_canvas_height;
+  /** The width of the canvas in physical pixels. */
+  int m_canvas_width;
+  /** The height of the canvas in physical pixels. */
+  int m_canvas_height;
 
   int xr_margin;  // chart scroll margins, control cursor, etc.
   int xl_margin;
@@ -748,6 +1032,8 @@ private:
 
   wxTimer m_RolloverPopupTimer;
 
+  wxTimer m_VPMovementTimer;
+
   int m_wheelzoom_stop_oneshot;
   int m_last_wheel_dir;
   wxStopWatch m_wheelstopwatch;
@@ -777,6 +1063,21 @@ private:
   emboss_data *m_pEM_OverZoom;
   //      emboss_data *m_pEM_CM93Offset;  // Flav
 
+  /**
+   * Physical pixels per meter at chart center, accounting for latitude
+   * distortion.
+   *
+   * This represents the true displayed scale at the chart center, calculated by
+   * measuring the screen distance in pixels between two points of known
+   * geographic distance. Unlike GetVPScale() which is the requested scale, this
+   * accounts for projection distortions, particularly in Mercator projection
+   * where scale varies with latitude.
+   *
+   * @note Key scale relationships:
+   * - GetVPScale() is the scale the user asks for.
+   * - m_true_scale_ppm is the actual scale after projection and latitude
+   * effects.
+   */
   double m_true_scale_ppm;
 
   ownship_state_t m_ownship_state;
@@ -863,6 +1164,11 @@ private:
   double m_panspeed;
   bool m_bmouse_key_mod;
   double m_zoom_factor, m_rotation_speed;
+  /**
+   * Timer/counter for smooth movement operations (panning, zooming, rotating).
+   * Represents the remaining duration (in milliseconds) for which movement
+   * should continue. Used to control animation steps.
+   */
   int m_mustmove;
 
   wxDateTime m_last_movement_time;
@@ -931,8 +1237,58 @@ private:
   double m_sector_glat, m_sector_glon;
   std::vector<s57Sector_t> m_sectorlegsVisible;
   bool m_bShowVisibleSectors;
+  /** Physical to logical pixel ratio for the display. */
   double m_displayScale;
+  bool m_show_focus_bar;
 
+  double m_panx_target_final;
+  double m_pany_target_final;
+  double m_panx_target_now;
+  double m_pany_target_now;
+
+  double m_start_lat, m_start_lon;
+  double m_target_lat, m_target_lon;
+  double m_run_lat, m_run_lon;
+  bool m_timed_move_vp_active;
+  int m_timedVP_step;
+  int m_stvpc;
+
+  double meters_to_shift = 0;
+  double dir_to_shift = 0;
+
+  // Chart drag inertia support
+  wxTimer m_chart_drag_inertia_timer;
+  void OnChartDragInertiaTimer(wxTimerEvent &event);
+
+  uint64_t m_last_drag_time;
+  int m_chart_drag_total_x;
+  int m_chart_drag_total_y;
+  double m_chart_drag_total_time;
+  double m_chart_drag_velocity_x;
+  double m_chart_drag_velocity_y;
+  wxLongLong m_chart_drag_inertia_time;
+  wxLongLong m_chart_drag_inertia_start_time;
+  bool m_chart_drag_inertia_active;
+  double m_last_elapsed;
+  std::vector<int> m_drag_vec_x;
+  std::vector<int> m_drag_vec_y;
+  std::vector<double> m_drag_vec_t;
+  int m_inertia_last_drag_x;
+  int m_inertia_last_drag_y;
+
+  // For Jump animation
+  wxTimer m_easeTimer;
+  wxLongLong m_animationStart;
+  wxLongLong m_animationDuration;  // e.g. 300 ms
+  double m_startLat, m_startLon, m_startScale;
+  double m_endLat, m_endLon, m_endScale;
+  bool m_animationActive;
+  void OnJumpEaseTimer(wxTimerEvent &event);
+  bool StartSmoothJump(double lat, double lon, double scale_ppm);
+
+  NotificationButton *m_notification_button;
+  NotificationsList *m_NotificationsList;
+  ObservableListener evt_notificationlist_change_listener;
   DECLARE_EVENT_TABLE()
 };
 
