@@ -38,12 +38,14 @@
 #include "model/comm_appmsg_bus.h"
 #include "bbox.h"
 #include "comm_overflow_dlg.h"
+#include "connections_dlg.h"
 #include "color_handler.h"
+#include "data_monitor.h"
 #include "gui_lib.h"
 #include "load_errors_dlg.h"
 #include "observable_evtvar.h"
-#include "ocpn_print.h"
 #include "s52s57.h"
+#include "s57registrar_mgr.h"
 #include "SencManager.h"
 #include "displays.h"
 
@@ -69,7 +71,6 @@ class ocpnFloatingToolbarDialog;
 class OCPN_MsgEvent;
 class options;
 class Track;
-// class OCPN_ThreadMessageEvent;
 class wxHtmlWindow;
 class ArrayOfCDI;
 
@@ -123,62 +124,76 @@ class ocpnToolBarSimple;
 class OCPN_DataStreamEvent;
 class AisTargetData;
 
-bool isSingleChart(ChartBase *chart);
+bool ShowNavWarning();
 
-class OCPN_ThreadMessageEvent : public wxEvent {
-public:
-  OCPN_ThreadMessageEvent(wxEventType commandType = wxEVT_NULL, int id = 0);
-  ~OCPN_ThreadMessageEvent();
-
-  // accessors
-  [[maybe_unused]] void SetSString(std::string string) { m_string = string; }
-  std::string GetSString() { return m_string; }
-
-  // required for sending with wxPostEvent()
-  wxEvent *Clone() const;
-
-private:
-  std::string m_string;
-};
+bool isSingleChart(ChartBase* chart);
 
+/**
+ * Main application frame. Top-level window frame for OpenCPN that manages
+ * overall application state, menus, toolbars, and child windows like chart
+ * canvases.
+ */
 class MyFrame : public wxFrame {
 public:
-  MyFrame(wxFrame *frame, const wxString &title, const wxPoint &pos,
-          const wxSize &size, long style);
+  MyFrame(wxFrame* frame, const wxString& title, const wxPoint& pos,
+          const wxSize& size, long style);
 
   ~MyFrame();
 
   int GetApplicationMemoryUse(void);
 
-  void OnEraseBackground(wxEraseEvent &event);
-  void OnMaximize(wxMaximizeEvent &event);
-  void OnCloseWindow(wxCloseEvent &event);
-  void OnExit(wxCommandEvent &event);
-  void OnSize(wxSizeEvent &event);
-  void OnMove(wxMoveEvent &event);
-  void OnInitTimer(wxTimerEvent &event);
-  void OnFrameTimer1(wxTimerEvent &event);
+  void OnEraseBackground(wxEraseEvent& event);
+  void OnMaximize(wxMaximizeEvent& event);
+  void OnCloseWindow(wxCloseEvent& event);
+  void OnExit(wxCommandEvent& event);
+  void OnSize(wxSizeEvent& event);
+  void OnMove(wxMoveEvent& event);
+  void OnInitTimer(wxTimerEvent& event);
+
+  /**
+   * Main application timer handler called approximately once per second.
+   *
+   * Key responsibilities include anchor watch monitoring, GPS position
+   * updates to plugins, toolbar management, and chart database refreshes.
+   *
+   * @param event The wxTimerEvent that triggered this handler
+   */
+  void OnFrameTimer1(wxTimerEvent& event);
+
   bool DoChartUpdate(void);
-  void OnEvtPlugInMessage(OCPN_MsgEvent &event);
-  void OnMemFootTimer(wxTimerEvent &event);
-  void OnRecaptureTimer(wxTimerEvent &event);
-  void OnSENCEvtThread(OCPN_BUILDSENC_ThreadEvent &event);
-  void OnIconize(wxIconizeEvent &event);
-  void OnBellsFinished(wxCommandEvent &event);
+  void OnEvtPlugInMessage(OCPN_MsgEvent& event);
+  void OnMemFootTimer(wxTimerEvent& event);
+  void OnRecaptureTimer(wxTimerEvent& event);
+  void OnSENCEvtThread(OCPN_BUILDSENC_ThreadEvent& event);
+  void OnIconize(wxIconizeEvent& event);
+  void OnBellsFinished(wxCommandEvent& event);
+
+  /**
+   * High-frequency timer handler running at 10Hz for smooth navigation updates.
+   *
+   * This timer event handler runs at 10Hz to provide smooth, high-frequency
+   * updates for navigation display and position estimation. It serves two
+   * primary purposes: handling compass rose rotation in non-North-Up modes and
+   * providing interpolated position/heading estimates when 10Hz updates are
+   * enabled.
+   *
+   * @param event The wxTimerEvent that triggered this handler (100ms interval)
+   */
+  void OnFrameTenHzTimer(wxTimerEvent& event);
 
 #ifdef wxHAS_POWER_EVENTS
-  void OnSuspending(wxPowerEvent &event);
-  void OnSuspended(wxPowerEvent &event);
-  void OnSuspendCancel(wxPowerEvent &event);
-  void OnResume(wxPowerEvent &event);
+  void OnSuspending(wxPowerEvent& event);
+  void OnSuspended(wxPowerEvent& event);
+  void OnSuspendCancel(wxPowerEvent& event);
+  void OnResume(wxPowerEvent& event);
 #endif  // wxHAS_POWER_EVENTS
 
-  void RefreshCanvasOther(ChartCanvas *ccThis);
+  void RefreshCanvasOther(ChartCanvas* ccThis);
   void UpdateAllFonts(void);
   void PositionConsole(void);
-  void OnToolLeftClick(wxCommandEvent &event);
-  void DoStackUp(ChartCanvas *cc);
-  void DoStackDown(ChartCanvas *cc);
+  void OnToolLeftClick(wxCommandEvent& event);
+  void DoStackUp(ChartCanvas* cc);
+  void DoStackDown(ChartCanvas* cc);
   void selectChartDisplay(int type, int family);
   void applySettingsString(wxString settings);
   void setStringVP(wxString VPS);
@@ -187,15 +202,16 @@ public:
   void CancelAllMouseRoute();
   void InvalidateAllQuilts();
 
-  void SetUpMode(ChartCanvas *cc, int mode);
+  void SetUpMode(ChartCanvas* cc, int mode);
 
-  ChartCanvas *GetPrimaryCanvas();
-  ChartCanvas *GetFocusCanvas();
+  ChartCanvas* GetPrimaryCanvas();
+  ChartCanvas* GetFocusCanvas();
 
-  void DoStackDelta(ChartCanvas *cc, int direction);
+  void DoStackDelta(ChartCanvas* cc, int direction);
   void DoSettings(void);
-  void SwitchKBFocus(ChartCanvas *pCanvas);
-  ChartCanvas *GetCanvasUnderMouse();
+  void DoSettingsNew(void);
+  void SwitchKBFocus(ChartCanvas* pCanvas);
+  ChartCanvas* GetCanvasUnderMouse();
   int GetCanvasIndexUnderMouse();
 
   bool DropMarker(bool atOwnShip = true);
@@ -203,10 +219,10 @@ public:
   void TriggerRecaptureTimer();
   bool SetGlobalToolbarViz(bool viz);
 
-  void MouseEvent(wxMouseEvent &event);
-  void CenterView(ChartCanvas *cc, const LLBBox &bbox);
+  void MouseEvent(wxMouseEvent& event);
+  void CenterView(ChartCanvas* cc, const LLBBox& bbox);
 
-  void JumpToPosition(ChartCanvas *cc, double lat, double lon, double scale);
+  void JumpToPosition(ChartCanvas* cc, double lat, double lon, double scale);
 
   void ProcessCanvasResize(void);
 
@@ -214,45 +230,47 @@ public:
   void ApplyGlobalSettings(bool bnewtoolbar);
   void RegisterGlobalMenuItems();
   void UpdateGlobalMenuItems();
-  void UpdateGlobalMenuItems(ChartCanvas *cc);
-  int DoOptionsDialog();
-  bool ProcessOptionsDialog(int resultFlags, ArrayOfCDI *pNewDirArray);
+  void UpdateGlobalMenuItems(ChartCanvas* cc);
+  void DoOptionsDialog();
+  void ProcessOptionsDialog(int resultFlags, ArrayOfCDI* pNewDirArray);
+  void PrepareOptionsClose(options* settings, int settings_return_value);
+
   void DoPrint(void);
-  void ToggleDataQuality(ChartCanvas *cc);
-  void TogglebFollow(ChartCanvas *cc);
+  void ToggleDataQuality(ChartCanvas* cc);
+  void TogglebFollow(ChartCanvas* cc);
   void ToggleFullScreen();
-  void ToggleChartBar(ChartCanvas *cc);
-  void SetbFollow(ChartCanvas *cc);
-  void ClearbFollow(ChartCanvas *cc);
-  void ToggleChartOutlines(ChartCanvas *cc);
-  void ToggleENCText(ChartCanvas *cc);
-  void ToggleSoundings(ChartCanvas *cc);
+  void ToggleChartBar(ChartCanvas* cc);
+  void SetbFollow(ChartCanvas* cc);
+  void ClearbFollow(ChartCanvas* cc);
+  void ToggleChartOutlines(ChartCanvas* cc);
+  void ToggleENCText(ChartCanvas* cc);
+  void ToggleSoundings(ChartCanvas* cc);
 #if 0
   void ToggleRocks(void);
 #endif
-  bool ToggleLights(ChartCanvas *cc);
-  void ToggleAnchor(ChartCanvas *cc);
-  void ToggleAISDisplay(ChartCanvas *cc);
-  void ToggleAISMinimizeTargets(ChartCanvas *cc);
+  bool ToggleLights(ChartCanvas* cc);
+  void ToggleAnchor(ChartCanvas* cc);
+  void ToggleAISDisplay(ChartCanvas* cc);
+  void ToggleAISMinimizeTargets(ChartCanvas* cc);
 
   void ToggleTestPause(void);
   void TrackOn(void);
-  void SetENCDisplayCategory(ChartCanvas *cc, enum _DisCat nset);
-  void ToggleNavobjects(ChartCanvas *cc);
+  void SetENCDisplayCategory(ChartCanvas* cc, enum _DisCat nset);
+  void ToggleNavobjects(ChartCanvas* cc);
 
-  Track *TrackOff(bool do_add_point = false);
+  Track* TrackOff(bool do_add_point = false);
   void TrackDailyRestart(void);
   bool ShouldRestartTrack();
   void ToggleColorScheme();
   void SetMenubarItemState(int item_id, bool state);
   void SetMasterToolbarItemState(int tool_id, bool state);
 
-  void SetToolbarItemBitmaps(int tool_id, wxBitmap *bitmap,
-                             wxBitmap *bmpDisabled);
+  void SetToolbarItemBitmaps(int tool_id, wxBitmap* bitmap,
+                             wxBitmap* bmpDisabled);
   void SetToolbarItemSVG(int tool_id, wxString normalSVGfile,
                          wxString rolloverSVGfile, wxString toggledSVGfile);
-  void ToggleQuiltMode(ChartCanvas *cc);
-  void UpdateControlBar(ChartCanvas *cc);
+  void ToggleQuiltMode(ChartCanvas* cc);
+  void UpdateControlBar(ChartCanvas* cc);
 
   void SubmergeAllCanvasToolbars(void);
   void SurfaceAllCanvasToolbars(void);
@@ -262,13 +280,15 @@ public:
 
   void RefreshGroupIndices(void);
 
-  double GetBestVPScale(ChartBase *pchart);
+  double GetBestVPScale(ChartBase* pchart);
+
+  DataMonitor* GetDataMonitor() const { return m_data_monitor; }
 
   ColorScheme GetColorScheme();
   void SetAndApplyColorScheme(ColorScheme cs);
 
-  void OnFrameTCTimer(wxTimerEvent &event);
-  void OnFrameCOGTimer(wxTimerEvent &event);
+  void OnFrameTCTimer(wxTimerEvent& event);
+  void OnFrameCOGTimer(wxTimerEvent& event);
 
   void HandleBasicNavMsg(std::shared_ptr<const BasicNavDataMsg> msg);
   void HandleGPSWatchdogMsg(std::shared_ptr<const GPSWatchdogMsg> msg);
@@ -283,11 +303,11 @@ public:
   void TouchAISActive(void);
   void UpdateAISTool(void);
 
-  void ActivateAISMOBRoute(const AisTargetData *ptarget);
-  void UpdateAISMOBRoute(const AisTargetData *ptarget);
+  void ActivateAISMOBRoute(const AisTargetData* ptarget);
+  void UpdateAISMOBRoute(const AisTargetData* ptarget);
 
-  wxStatusBar *m_pStatusBar;
-  wxMenuBar *m_pMenuBar;
+  wxStatusBar* m_pStatusBar;
+  wxMenuBar* m_pMenuBar;
   int nBlinkerTick;
   bool m_bTimeIsSet;
 
@@ -300,13 +320,14 @@ public:
   wxTimer FrameCOGTimer;
   wxTimer MemFootTimer;
   wxTimer m_resizeTimer;
+  wxTimer FrameTenHzTimer;
 
   int m_BellsToPlay;
   wxTimer BellsTimer;
 
   //      PlugIn support
   int GetNextToolbarToolId() { return m_next_available_plugin_tool_id; }
-  void RequestNewToolbarArgEvent(wxCommandEvent &WXUNUSED(event)) {
+  void RequestNewToolbarArgEvent(wxCommandEvent& WXUNUSED(event)) {
     return RequestNewMasterToolbar();
   }
   void RequestNewToolbars(bool bforcenew = false);
@@ -315,9 +336,9 @@ public:
   void UpdateGPSCompassStatusBoxes(bool b_force_new = false);
   void UpdateRotationState(double rotation);
 
-  bool UpdateChartDatabaseInplace(ArrayOfCDI &DirArray, bool b_force,
+  bool UpdateChartDatabaseInplace(ArrayOfCDI& DirArray, bool b_force,
                                   bool b_prog,
-                                  const wxString &ChartListFileName);
+                                  const wxString& ChartListFileName);
 
   bool m_bdefer_resize;
   wxSize m_defer_size;
@@ -330,7 +351,7 @@ public:
   void ReloadAllVP();
   void SetCanvasSizes(wxSize frameSize);
 
-  ocpnToolBarSimple *CreateMasterToolbar();
+  ocpnToolBarSimple* CreateMasterToolbar();
   void RequestNewMasterToolbar(bool bforcenew = true);
   bool CheckAndAddPlugInTool();
   bool AddDefaultPositionPlugInTools();
@@ -338,6 +359,9 @@ public:
   void NotifyChildrenResize(void);
   void UpdateCanvasConfigDescriptors();
   void ScheduleSettingsDialog();
+  void ScheduleSettingsDialogNew();
+  void ScheduleDeleteSettingsDialog();
+  void ScheduleReconfigAndSettingsReload(bool reload, bool new_dialog);
   static void RebuildChartDatabase();
   void PositionIENCToolbar();
 
@@ -345,9 +369,18 @@ public:
   void InitApiListeners();
   void ReleaseApiListeners();
   void UpdateStatusBar(void);
+  void ConfigureStatusBar();
 
 private:
+  void ProcessUnitTest();
+  void ProcessQuitFlag();
+  void ProcessDeferredTrackOn();
+  void SendFixToPlugins();
+  void ProcessAnchorWatch();
+  void ProcessLogAndBells();
+  void CalculateCOGAverage();
   void CheckToolbarPosition();
+
   void ODoSetSize(void);
   void DoCOGSet(void);
 
@@ -359,7 +392,7 @@ private:
 
   bool ScrubGroupArray();
 
-  void OnToolbarAnimateTimer(wxTimerEvent &event);
+  void OnToolbarAnimateTimer(wxTimerEvent& event);
   bool CollapseGlobalToolbar();
 
   int m_StatusBarFieldCount;
@@ -367,6 +400,7 @@ private:
   wxDateTime m_MMEAeventTime;
   unsigned long m_ulLastNMEATicktime;
   int m_tick_idx;
+  wxDateTime m_fix_start_time;
 
   wxString m_last_reported_chart_name;
   wxString m_last_reported_chart_pubdate;
@@ -382,9 +416,17 @@ private:
   int m_ChartUpdatePeriod;
   bool m_last_bGPSValid;
   bool m_last_bVelocityValid;
+  double m_last_hdt;
 
   wxString prev_locale;
 
+  /**
+   * The last time basic navigational data was received, or 0 if no data
+   * has been received.
+   *
+   * @todo Change time_t to wxLongLong, as time_t is susceptible to the
+   * year 2038 problem on 32-bit builds.
+   */
   time_t m_fixtime;
   bool b_autofind;
 
@@ -400,8 +442,15 @@ private:
   ObsListener m_on_raise_listener;
   ObsListener m_on_quit_listener;
   ObsListener m_routes_update_listener;
+  ObsListener m_evt_drv_msg_listener;
 
   CommOverflowDlg comm_overflow_dlg;
+  ConnectionsDlg* m_connections_dlg;
+  bool m_need_new_options;
+  wxArrayString pathArray;
+  double restoreScale[4];
+  unsigned int last_canvasConfig;
+  DataMonitor* m_data_monitor;
 
   DECLARE_EVENT_TABLE()
 };
