@@ -73,7 +73,7 @@
 #include "model/plugin_paths.h"
 #include "model/select.h"
 
-#include "AboutFrameImpl.h"
+#include "about_frame_impl.h"
 #include "about.h"
 #include "displays.h"
 #include "FontMgr.h"
@@ -158,7 +158,6 @@ extern bool g_bPermanentMOBIcon;
 extern float g_toolbar_scalefactor;
 
 extern options *g_options;
-extern bool g_boptionsactive;
 
 extern wxString *pInit_Chart_Dir;
 
@@ -206,7 +205,7 @@ extern wxArrayString g_locale_catalog_array;
 #endif
 extern int options_lastPage;
 extern AboutFrameImpl *g_pAboutDlg;
-extern about *g_pAboutDlgLegacy;
+extern About *g_pAboutDlgLegacy;
 extern wxColour g_colourTrackLineColour;
 extern int g_n_ownship_min_mm;
 
@@ -580,8 +579,8 @@ void OCPNPlatform::Initialize_2(void) {
 
   // Set the default Import/Export directory for A11+
   if (g_Android_SDK_Version >= 30) {
-    if (!g_gpx_path.StartsWith(androidGetDownloadDirectory())) {
-      g_gpx_path = androidGetDownloadDirectory();
+    if (!g_gpx_path.StartsWith(androidGetDocumentsDirectory())) {
+      g_gpx_path = androidGetDocumentsDirectory();
     }
   }
 
@@ -994,6 +993,11 @@ void OCPNPlatform::SetLocaleSearchPrefixes(void) {
   if (!wxGetEnv(_T("OPENCPN_PREFIX"), &locale_location)) {
     locale_location = _T("/usr/local");
   }
+
+  if (g_bportable) {
+    locale_location = g_Platform->GetHomeDir();
+  }
+
   wxFileName location;
   location.AssignDir(locale_location);
   location.AppendDir(_T("share"));
@@ -1002,14 +1006,14 @@ void OCPNPlatform::SetLocaleSearchPrefixes(void) {
   wxLocale::AddCatalogLookupPathPrefix(locale_location);
 
   // And then for managed plugins
-  std::string dir = PluginPaths::getInstance()->UserDatadir();
+  std::string dir = PluginPaths::GetInstance()->UserDatadir();
   wxString managed_locale_location(dir + "/locale");
   wxLocale::AddCatalogLookupPathPrefix(managed_locale_location);
 #endif
 
 #ifdef __WXOSX__
   std::string macDir =
-      PluginPaths::getInstance()->Homedir() +
+      PluginPaths::GetInstance()->Homedir() +
       "/Library/Application Support/OpenCPN/Contents/Resources";
   wxString Mac_managed_locale_location(macDir);
   wxLocale::AddCatalogLookupPathPrefix(Mac_managed_locale_location);
@@ -1244,7 +1248,7 @@ void OCPNPlatform::SetDefaultOptions(void) {
     pConfig->SetPath(_T ( "/Settings/GlobalState" ));
     pConfig->Write(_T ( "bShowS57Text" ), true);
     pConfig->Write(_T ( "bShowS57ImportantTextOnly" ), false);
-    pConfig->Write(_T ( "nDisplayCategory" ), (int)(_DisCat)STANDARD);
+    pConfig->Write(_T ( "nDisplayCategory" ), (int)(_DisCat)OTHER);
     pConfig->Write(_T ( "nSymbolStyle" ), (int)(_LUPname)PAPER_CHART);
     pConfig->Write(_T ( "nBoundaryStyle" ), (int)(_LUPname)PLAIN_BOUNDARIES);
 
@@ -1355,7 +1359,7 @@ void OCPNPlatform::SetDefaultOptions(void) {
   ConnectionParams *new_params = new ConnectionParams(sGPS);
 
   new_params->bEnabled = true;
-  TheConnectionParams()->Add(new_params);
+  TheConnectionParams().push_back(new_params);
 
   g_default_font_facename = _T("Roboto");
 
@@ -1417,8 +1421,8 @@ void OCPNPlatform::SetDefaultOptions(void) {
 
 //      Setup global options on upgrade detected
 //      The global config object (pConfig) has already been loaded, so updates
-//      here override values set by config Direct updates to config for next
-//      boot are also allowed
+//      here override values set by config.
+//      Direct updates to config for next boot are also supported.
 
 void OCPNPlatform::SetUpgradeOptions(wxString vNew, wxString vOld) {
 #ifdef __ANDROID__
@@ -1522,6 +1526,15 @@ void OCPNPlatform::SetUpgradeOptions(wxString vNew, wxString vOld) {
       wxFileName ft(tide);
       if (ft.FileExists()) TideCurrentDataSet.push_back(TCDS_temp[i]);
     }
+
+    //  Force default (baked-in) values for CompatOS
+    // Thus providing for the case when some core dependent elements
+    // are updated. (e.g. flatpak SDK 22.08->24.08->???)
+    g_compatOS = "";
+    g_compatOsVersion = "";
+    pConfig->SetPath(_T ( "/Settings" ));
+    pConfig->Write(_T ( "CompatOS" ), g_compatOS);
+    pConfig->Write(_T ( "CompatOsVersion" ), g_compatOsVersion);
   }
 }
 
@@ -2175,7 +2188,10 @@ bool OCPNPlatform::AllowAlertDialog(const wxString &class_name) {
   }
 
   // qDebug() << "AllowAlertDialog" << g_boptionsactive << g_running << nTLW;
-  return (g_running && !g_boptionsactive && (nTLW <= 4));
+  if (g_options)
+    return (g_running && !g_options->IsShown() && (nTLW <= 4));
+  else
+    return (g_running && (nTLW <= 4));
 
 #else
   return true;
@@ -2234,54 +2250,6 @@ bool OCPNPlatform::isPlatformCapable(int flag) {
 #endif
 }
 
-void OCPNPlatform::DoHelpDialog(void) {
-#ifndef __ANDROID__
-  if (!g_pAboutDlg) {
-    g_pAboutDlg = new AboutFrameImpl(gFrame);
-  } else {
-    g_pAboutDlg->SetFocus();
-  }
-  g_pAboutDlg->Show();
-
-#else
-  if (!g_pAboutDlgLegacy)
-    g_pAboutDlgLegacy = new about(gFrame, GetSharedDataDir());
-  else
-    g_pAboutDlgLegacy->SetFocus();
-  g_pAboutDlgLegacy->Show();
-
-#endif
-}
-
-void OCPNPlatform::LaunchLocalHelp(void) {
-#ifdef __ANDROID__
-  androidLaunchHelpView();
-#else
-  wxString def_lang_canonical = _T("en_US");
-
-#if wxUSE_XLOCALE
-  if (plocale_def_lang)
-    def_lang_canonical = plocale_def_lang->GetCanonicalName();
-#endif
-
-  wxString help_locn = g_Platform->GetSharedDataDir() + _T("doc/help_");
-
-  wxString help_try = help_locn + def_lang_canonical + _T(".html");
-
-  if (!::wxFileExists(help_try)) {
-    help_try = help_locn + _T("en_US") + _T(".html");
-
-    if (!::wxFileExists(help_try)) {
-      help_try = help_locn + _T("web") + _T(".html");
-    }
-
-    if (!::wxFileExists(help_try)) return;
-  }
-
-  wxLaunchDefaultBrowser(wxString(_T("file:///")) + help_try);
-#endif
-}
-
 void OCPNPlatform::platformLaunchDefaultBrowser(wxString URL) {
 #ifdef __ANDROID__
   androidLaunchBrowser(URL);
