@@ -44,11 +44,13 @@
 #include "model/config_vars.h"
 #include "model/cutil.h"
 #include "model/georef.h"
+#include "model/gui.h"
 #include "model/mdns_cache.h"
-#include "model/mDNS_query.h"
+#include "model/mdns_query.h"
 #include "model/nav_object_database.h"
 #include "model/own_ship.h"
 #include "model/own_ship.h"
+#include "model/plugin_comm.h"
 #include "model/route.h"
 #include "model/routeman.h"
 #include "model/select.h"
@@ -75,6 +77,7 @@
 #include "route_point_gui.h"
 #include "RoutePropDlgImpl.h"
 #include "s52plib.h"
+#include "s52s57.h"
 #include "s57chart.h"  // for ArrayOfS57Obj
 #include "SendToGpsDlg.h"
 #include "SendToPeerDlg.h"
@@ -85,6 +88,7 @@
 #include "track_gui.h"
 #include "TrackPropDlg.h"
 #include "undo.h"
+#include "model/navobj_db.h"
 
 #ifdef __ANDROID__
 #include "androidUTIL.h"
@@ -205,6 +209,9 @@ enum {
   ID_DEF_MENU_CURRENTINFO,
   ID_DEF_ZERO_XTE,
 
+  ID_DEF_MENU_DEBUG,
+  ID_DGB_MENU_NMEA_WINDOW,
+
   ID_DEF_MENU_GROUPBASE,  // Must be last entry, as chart group identifiers are
                           // created dynamically
 
@@ -226,9 +233,8 @@ CanvasMenuHandler::CanvasMenuHandler(ChartCanvas *parentCanvas,
                                      Route *selectedRoute, Track *selectedTrack,
                                      RoutePoint *selectedPoint,
                                      int selectedAIS_MMSI,
-                                     void *selectedTCIndex)
-
-{
+                                     void *selectedTCIndex, wxWindow *nmea_log)
+    : m_nmea_log(nmea_log) {
   parent = parentCanvas;
   m_pSelectedRoute = selectedRoute;
   m_pSelectedTrack = selectedTrack;
@@ -282,7 +288,7 @@ void CanvasMenuHandler::MenuAppend1(wxMenu *menu, int id, wxString label) {
 #endif
 
 #ifdef __ANDROID__
-  wxFont sFont = GetOCPNGUIScaledFont(_T("Menu"));
+  wxFont sFont = GetOCPNGUIScaledFont(_("Menu"));
   item->SetFont(sFont);
 #endif
 
@@ -320,6 +326,8 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
 #else
   wxMenu *subMenuRedo = new wxMenu("Redo...Ctrl-Y");
 #endif
+  wxMenu *subMenuDebug = new wxMenu("");
+  MenuAppend1(subMenuDebug, ID_DGB_MENU_NMEA_WINDOW, _("Show Data Monitor"));
 
   wxMenu *menuFocus = contextMenu;  // This is the one that will be shown
 
@@ -511,8 +519,9 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
       MenuAppend1(contextMenu, ID_DEF_MENU_MOVE_BOAT_HERE, _("Move Boat Here"));
   }
 
-  if (!g_bBasicMenus &&
-      (!(g_pRouteMan->GetpActiveRoute() || (seltype & SELTYPE_MARKPOINT))))
+  if (!g_bBasicMenus && !g_pRouteMan->GetpActiveRoute() &&
+      (!(seltype & SELTYPE_MARKPOINT) ||
+       (m_pFoundRoutePoint && m_pFoundRoutePoint->m_bIsInLayer)))
     MenuAppend1(contextMenu, ID_DEF_MENU_GOTO_HERE, _("Navigate To Here"));
 
   if (!g_bBasicMenus)
@@ -701,6 +710,12 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
       }
     }
   }
+  if (g_enable_root_menu_debug) {
+    wxMenuItem *subItemDebug =
+        contextMenu->AppendSubMenu(subMenuDebug, _("Debug"));
+    if (g_btouch) contextMenu->AppendSeparator();
+    SetMenuItemFont1(subItemDebug);
+  }
 
   if (seltype & SELTYPE_ROUTESEGMENT) {
     if (!g_bBasicMenus && m_pSelectedRoute) {
@@ -957,6 +972,11 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
     // Eventually set this menu as the "focused context menu"
     if (menuFocus != menuAIS) menuFocus = menuWaypoint;
   }
+
+  //  Add plugin context items to the correct menu
+  AddPluginContextMenuItems(contextMenu, menuRoute, menuTrack, menuWaypoint,
+                            menuAIS);
+
   /*add the relevant submenus*/
   enum { WPMENU = 1, TKMENU = 2, RTMENU = 4, MMMENU = 8 };
   int sub_menu = 0;
@@ -965,6 +985,7 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
         global_color_scheme != GLOBAL_COLOR_SCHEME_NIGHT) {
       menuFocus->AppendSeparator();
     }
+
     wxMenuItem *subMenu1;
     if (menuWaypoint && menuFocus != menuWaypoint) {
       subMenu1 =
@@ -1013,6 +1034,40 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
                 _("Show Current Information"));
   }
 
+  //        Invoke the correct focused drop-down menu
+
+#ifdef __ANDROID__
+  androidEnableBackButton(false);
+  androidEnableOptionsMenu(false);
+
+  setMenuStyleSheet(menuRoute, GetOCPNGUIScaledFont(_("Menu")));
+  setMenuStyleSheet(menuWaypoint, GetOCPNGUIScaledFont(_("Menu")));
+  setMenuStyleSheet(menuTrack, GetOCPNGUIScaledFont(_("Menu")));
+  setMenuStyleSheet(menuAIS, GetOCPNGUIScaledFont(_("Menu")));
+#endif
+
+  parent->PopupMenu(menuFocus, x, y);
+
+#ifdef __ANDROID__
+  androidEnableBackButton(true);
+  androidEnableOptionsMenu(true);
+#endif
+
+  /* Cleanup if necessary.
+  Do not delete menus witch are submenu as they will be deleted by their parent
+  menu. This could create a crash*/
+  delete menuAIS;
+  if (!(sub_menu & MMMENU)) delete contextMenu;
+  if (!(sub_menu & RTMENU)) delete menuRoute;
+  if (!(sub_menu & TKMENU)) delete menuTrack;
+  if (!(sub_menu & WPMENU)) delete menuWaypoint;
+}
+
+void CanvasMenuHandler::AddPluginContextMenuItems(wxMenu *contextMenu,
+                                                  wxMenu *menuRoute,
+                                                  wxMenu *menuTrack,
+                                                  wxMenu *menuWaypoint,
+                                                  wxMenu *menuAIS) {
   // Give the plugins a chance to update their menu items
   g_pi_manager->PrepareAllPluginContextMenus();
 
@@ -1026,48 +1081,36 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
 
     wxMenu *submenu = NULL;
     if (pimis->pmenu_item->GetSubMenu()) {
+      // Create a clone of the submenu referenced in the PlugInMenuItemContainer
+      auto submenu_proto = pimis->pmenu_item->GetSubMenu();
       submenu = new wxMenu();
       const wxMenuItemList &items =
           pimis->pmenu_item->GetSubMenu()->GetMenuItems();
       for (wxMenuItemList::const_iterator it = items.begin(); it != items.end();
            ++it) {
-        int id = -1;
-        for (unsigned int j = 0; j < item_array.GetCount(); j++) {
-          PlugInMenuItemContainer *pimis = item_array[j];
-          if (pimis->pmenu_item == *it) id = pimis->id;
-        }
-
-        wxMenuItem *pmi = new wxMenuItem(submenu, id,
-#if wxCHECK_VERSION(3, 0, 0)
-                                         (*it)->GetItemLabelText(),
-#else
-                                         (*it)->GetLabel(),
-#endif
-                                         (*it)->GetHelp(), (*it)->GetKind());
+        int id = (*it)->GetId();
+        wxMenuItem *psmi =
+            new wxMenuItem(submenu, id, (*it)->GetItemLabelText(),
+                           (*it)->GetHelp(), (*it)->GetKind());
 
 #ifdef __WXMSW__
-        pmi->SetFont(m_scaledFont);
+        psmi->SetFont(m_scaledFont);
 #endif
 
 #ifdef __ANDROID__
         wxFont sFont = GetOCPNGUIScaledFont(_("Menu"));
-        pmi->SetFont(sFont);
+        psmi->SetFont(sFont);
 #endif
 
-        PrepareMenuItem(pmi);
-        submenu->Append(pmi);
-        pmi->Check((*it)->IsChecked());
+        PrepareMenuItem(psmi);
+        submenu->Append(psmi);
+        psmi->Check((*it)->IsChecked());
       }
     }
 
-    wxMenuItem *pmi = new wxMenuItem(contextMenu, pimis->id,
-#if wxCHECK_VERSION(3, 0, 0)
-                                     pimis->pmenu_item->GetItemLabelText(),
-#else
-                                     pimis->pmenu_item->GetLabel(),
-#endif
-                                     pimis->pmenu_item->GetHelp(),
-                                     pimis->pmenu_item->GetKind(), submenu);
+    wxMenuItem *pmi = new wxMenuItem(
+        contextMenu, pimis->id, pimis->pmenu_item->GetItemLabelText(),
+        pimis->pmenu_item->GetHelp(), pimis->pmenu_item->GetKind(), submenu);
 #ifdef __WXMSW__
     pmi->SetFont(m_scaledFont);
 #endif
@@ -1094,34 +1137,6 @@ void CanvasMenuHandler::CanvasPopupMenu(int x, int y, int seltype) {
       dst->Enable(pimis->id, !pimis->b_grey);
     }
   }
-
-  //        Invoke the correct focused drop-down menu
-
-#ifdef __ANDROID__
-  androidEnableBackButton(false);
-  androidEnableOptionsMenu(false);
-
-  setMenuStyleSheet(menuRoute, GetOCPNGUIScaledFont(_T("Menu")));
-  setMenuStyleSheet(menuWaypoint, GetOCPNGUIScaledFont(_T("Menu")));
-  setMenuStyleSheet(menuTrack, GetOCPNGUIScaledFont(_T("Menu")));
-  setMenuStyleSheet(menuAIS, GetOCPNGUIScaledFont(_T("Menu")));
-#endif
-
-  parent->PopupMenu(menuFocus, x, y);
-
-#ifdef __ANDROID__
-  androidEnableBackButton(true);
-  androidEnableOptionsMenu(true);
-#endif
-
-  /* Cleanup if necessary.
-  Do not delete menus witch are submenu as they will be deleted by their parent
-  menu. This could create a crash*/
-  delete menuAIS;
-  if (!(sub_menu & MMMENU)) delete contextMenu;
-  if (!(sub_menu & RTMENU)) delete menuRoute;
-  if (!(sub_menu & TKMENU)) delete menuTrack;
-  if (!(sub_menu & WPMENU)) delete menuWaypoint;
 }
 
 void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
@@ -1211,7 +1226,8 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
                                        wxEmptyString, wxEmptyString);
       pWP->m_bIsolatedMark = true;  // This is an isolated mark
       pSelect->AddSelectableRoutePoint(zlat, zlon, pWP);
-      pConfig->AddNewWayPoint(pWP, -1);  // use auto next num
+      NavObj_dB::GetInstance().InsertRoutePoint(pWP);
+
       if (!RoutePointGui(*pWP).IsVisibleSelectable(this->parent))
         RoutePointGui(*pWP).ShowScaleWarningMessage(parent);
 
@@ -1319,11 +1335,12 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
             g_pRouteMan->GetRouteArrayContaining(m_pFoundRoutePoint);
         if (proute_array) {
           pWayPointMan->DestroyWaypoint(m_pFoundRoutePoint);
+          delete proute_array;
         } else {
           parent->undo->BeforeUndoableAction(
               Undo_DeleteWaypoint, m_pFoundRoutePoint, Undo_IsOrphanded,
               NULL /*m_pFoundPoint*/);
-          pConfig->DeleteWayPoint(m_pFoundRoutePoint);
+          NavObj_dB::GetInstance().DeleteRoutePoint(m_pFoundRoutePoint);
           pSelect->DeleteSelectablePoint(m_pFoundRoutePoint,
                                          SELTYPE_ROUTEPOINT);
           if (NULL != pWayPointMan)
@@ -1366,7 +1383,7 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
         wxJSONValue v;
         v[_T("GUID")] = guid;
         wxString msg_id(_T("OCPN_ANCHOR_WATCH_CLEARED"));
-        g_pi_manager->SendJSONMessageToAllPlugins(msg_id, v);
+        SendJSONMessageToAllPlugins(msg_id, v);
       }
       break;
     }
@@ -1398,7 +1415,7 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
         wxJSONValue v;
         v[_T("GUID")] = guid;
         wxString msg_id(_T("OCPN_ANCHOR_WATCH_SET"));
-        g_pi_manager->SendJSONMessageToAllPlugins(msg_id, v);
+        SendJSONMessageToAllPlugins(msg_id, v);
       }
       break;
     }
@@ -1438,8 +1455,7 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
       break;
     }
     case ID_DEF_MENU_AIS_QUERY: {
-      wxWindow *pwin = wxDynamicCast(parent, wxWindow);
-      ShowAISTargetQueryDialog(pwin, m_FoundAIS_MMSI);
+      ShowAISTargetQueryDialog(parent, m_FoundAIS_MMSI);
       break;
     }
 
@@ -1490,6 +1506,12 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
 
       break;
     }
+
+    case ID_DGB_MENU_NMEA_WINDOW:
+      m_nmea_log->Show();
+      m_nmea_log->Raise();
+      break;
+
     case ID_RT_MENU_REVERSE: {
       if (m_pSelectedRoute->m_bIsInLayer) break;
 
@@ -1502,7 +1524,7 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
         m_pSelectedRoute->Reverse(ask_return == wxID_YES);
         pSelect->AddAllSelectableRouteSegments(m_pSelectedRoute);
 
-        pConfig->UpdateRoute(m_pSelectedRoute);
+        NavObj_dB::GetInstance().UpdateRoute(m_pSelectedRoute);
 
         if (pRoutePropDialog && (pRoutePropDialog->IsShown())) {
           pRoutePropDialog->SetRouteAndUpdate(m_pSelectedRoute);
@@ -1543,23 +1565,16 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
     }
 
     case ID_RT_MENU_DELETE: {
-      int dlg_return = wxID_YES;
-      if (g_bConfirmObjectDelete) {
-        dlg_return = OCPNMessageBox(
-            parent, _("Are you sure you want to delete this route?"),
-            _("OpenCPN Route Delete"),
-            (long)wxYES_NO | wxCANCEL | wxYES_DEFAULT);
-      }
+      bool confirmed = RouteGui::OnDelete(parent);
 
-      if (dlg_return == wxID_YES) {
+      if (confirmed) {
         if (g_pRouteMan->GetpActiveRoute() == m_pSelectedRoute)
           g_pRouteMan->DeactivateRoute();
 
         if (m_pSelectedRoute->m_bIsInLayer) break;
 
-        if (!g_pRouteMan->DeleteRoute(m_pSelectedRoute,
-                                      NavObjectChanges::getInstance()))
-          break;
+        NavObj_dB::GetInstance().DeleteRoute(m_pSelectedRoute);
+        if (!g_pRouteMan->DeleteRoute(m_pSelectedRoute)) break;
 
         if (RouteManagerDialog::getInstanceFlag()) {
           if (pRouteManagerDialog && pRouteManagerDialog->IsShown())
@@ -1630,7 +1645,7 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
        }
        }
        */
-      pConfig->UpdateRoute(m_pSelectedRoute);
+      NavObj_dB::GetInstance().UpdateRoute(m_pSelectedRoute);
 
       if (pRoutePropDialog && (pRoutePropDialog->IsShown())) {
         pRoutePropDialog->SetRouteAndUpdate(m_pSelectedRoute, true);
@@ -1682,17 +1697,16 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
       m_pTail->CloneRoute(m_pSelectedRoute, m_SelectedIdx + splitMode,
                           m_pSelectedRoute->GetnPoints(), _("_B"), dupFirstWpt);
       pRouteList->Append(m_pHead);
-      pConfig->AddNewRoute(m_pHead);
+      NavObj_dB::GetInstance().InsertRoute(m_pHead);
 
       pRouteList->Append(m_pTail);
-      pConfig->AddNewRoute(m_pTail);
+      NavObj_dB::GetInstance().InsertRoute(m_pTail);
 
-      pConfig->DeleteConfigRoute(m_pSelectedRoute);
+      NavObj_dB::GetInstance().DeleteRoute(m_pSelectedRoute);
 
       pSelect->DeleteAllSelectableRoutePoints(m_pSelectedRoute);
       pSelect->DeleteAllSelectableRouteSegments(m_pSelectedRoute);
-      g_pRouteMan->DeleteRoute(m_pSelectedRoute,
-                               NavObjectChanges::getInstance());
+      g_pRouteMan->DeleteRoute(m_pSelectedRoute);
       pSelect->AddAllSelectableRouteSegments(m_pTail);
       pSelect->AddAllSelectableRoutePoints(m_pTail);
       pSelect->AddAllSelectableRouteSegments(m_pHead);
@@ -1727,7 +1741,7 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
         else {
           SendToGpsDlg dlg;
           dlg.SetWaypoint(m_pFoundRoutePoint);
-          wxFont fo = GetOCPNGUIScaledFont(_T("Dialog"));
+          wxFont fo = GetOCPNGUIScaledFont(_("Dialog"));
           dlg.SetFont(fo);
 
           dlg.Create(NULL, -1, _("Send to GPS") + _T( "..." ), _T(""));
@@ -1903,8 +1917,8 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
         if (m_pSelectedTrack == g_pActiveTrack)
           m_pSelectedTrack = parent->parent_frame->TrackOff();
         g_pAIS->DeletePersistentTrack(m_pSelectedTrack);
-        pConfig->DeleteConfigTrack(m_pSelectedTrack);
-
+        // pConfig->DeleteConfigTrack(m_pSelectedTrack);
+        NavObj_dB::GetInstance().DeleteTrack(m_pSelectedTrack);
         RoutemanGui(*g_pRouteMan).DeleteTrack(m_pSelectedTrack);
 
         if (TrackPropDlg::getInstanceFlag() && pTrackPropDialog &&
@@ -1986,14 +2000,53 @@ void CanvasMenuHandler::PopupMenuHandler(wxCommandEvent &event) {
 
       for (unsigned int i = 0; i < item_array.GetCount(); i++) {
         PlugInMenuItemContainer *pimis = item_array[i];
-        {
-          if (pimis->id == event.GetId()) {
-            if (pimis->m_pplugin)
-              pimis->m_pplugin->OnContextMenuItemCallback(pimis->id);
+        int target_id = pimis->id;
+
+        // Check submenus, if present.
+        if (pimis->pmenu_item->GetSubMenu()) {
+          const wxMenuItemList &items =
+              pimis->pmenu_item->GetSubMenu()->GetMenuItems();
+          for (wxMenuItemList::const_iterator it = items.begin();
+               it != items.end(); ++it) {
+            if ((*it)->GetId() == event.GetId()) {
+              target_id = (*it)->GetId();
+              break;
+            }
+          }
+        }
+
+        if (pimis->m_pplugin && (target_id == event.GetId())) {
+          if (pimis->extended) {
+            std::string object_type;
+            std::string object_ident;
+
+            if ((pimis->m_in_menu.IsSameAs("Waypoint")) && m_pFoundRoutePoint) {
+              object_type = "Waypoint";
+              object_ident = m_pFoundRoutePoint->m_GUID.ToStdString();
+            } else if ((pimis->m_in_menu.IsSameAs("Route")) &&
+                       m_pSelectedRoute) {
+              object_type = "Route";
+              object_ident = m_pSelectedRoute->m_GUID.ToStdString();
+            } else if ((pimis->m_in_menu.IsSameAs("Track")) &&
+                       m_pSelectedTrack) {
+              object_type = "Track";
+              object_ident = m_pSelectedTrack->m_GUID.ToStdString();
+            } else if ((pimis->m_in_menu.IsSameAs("AIS")) && m_FoundAIS_MMSI) {
+              object_type = "AIS";
+              wxString sAIS = wxString::Format("%d", m_FoundAIS_MMSI);
+              object_ident = sAIS.ToStdString();
+            }
+
+            opencpn_plugin_120 *ppi =
+                dynamic_cast<opencpn_plugin_120 *>(pimis->m_pplugin);
+            if (ppi)
+              ppi->OnContextMenuItemCallbackExt(target_id, object_ident,
+                                                object_type, zlat, zlon);
+          } else {
+            pimis->m_pplugin->OnContextMenuItemCallback(pimis->id);
           }
         }
       }
-
       break;
     }
   }  // switch
