@@ -108,11 +108,13 @@ using namespace std::literals::chrono_literals;
 #include "model/instance_check.h"
 #include "model/local_api.h"
 #include "model/logger.h"
-#include "model/mDNS_query.h"
-#include "model/mDNS_service.h"
+#include "model/mdns_query.h"
+#include "model/mdns_service.h"
 #include "model/multiplexer.h"
+#include "model/navobj_db.h"
 #include "model/nav_object_database.h"
 #include "model/navutil_base.h"
+#include "model/notification_manager.h"
 #include "model/own_ship.h"
 #include "model/plugin_handler.h"
 #include "model/route.h"
@@ -120,7 +122,7 @@ using namespace std::literals::chrono_literals;
 #include "model/select.h"
 #include "model/track.h"
 
-#include "AboutFrameImpl.h"
+#include "about_frame_impl.h"
 #include "about.h"
 #include "ais_info_gui.h"
 #include "AISTargetAlertDialog.h"
@@ -142,7 +144,6 @@ using namespace std::literals::chrono_literals;
 #include "Layer.h"
 #include "MarkInfo.h"
 #include "navutil.h"
-#include "NMEALogWindow.h"
 #include "observable.h"
 #include "ocpn_app.h"
 #include "OCPN_AUIManager.h"
@@ -194,7 +195,7 @@ void RedirectIOToConsole();
 #include "wiz_ui.h"
 
 const char *const kUsage =
-    R"""(Usage:
+    R"(Usage:
   opencpn -h | --help
   opencpn [-p] [-f] [-G] [-g] [-P] [-l <str>] [-u <num>] [-U] [-s] [GPX file ...]
   opencpn --remote [-R] | -q] | -e] |-o <str>]
@@ -225,7 +226,7 @@ Options manipulating already started opencpn
 
 Arguments:
   GPX  file                     GPX-formatted file with waypoints or routes.
-)""";
+)";
 
 //  comm event definitions
 wxDEFINE_EVENT(EVT_N2K_129029, wxCommandEvent);
@@ -262,7 +263,7 @@ bool g_bPauseTest;
 
 MyFrame *gFrame;
 
-ConsoleCanvas *console;
+APConsole *console;
 
 MyConfig *pConfig;
 ChartDB *ChartData;
@@ -339,12 +340,6 @@ ChartDummy *pDummyChart;
 
 ocpnStyle::StyleManager *g_StyleManager;
 
-// Global print data, to remember settings during the session
-wxPrintData *g_printData = (wxPrintData *)NULL;
-
-// Global page setup data
-wxPageSetupData *g_pageSetupData = (wxPageSetupData *)NULL;
-
 bool g_bShowOutlines;
 bool g_bShowDepthUnits;
 bool g_bDisplayGrid;  // Flag indicating weather the lat/lon grid should be
@@ -386,10 +381,6 @@ int gHDx_Watchdog;
 bool g_bDebugCM93;
 bool g_bDebugS57;
 
-bool g_bfilter_cogsog;
-int g_COGFilterSec = 1;
-int g_SOGFilterSec;
-
 int g_ChartUpdatePeriod;
 int g_SkewCompUpdatePeriod;
 
@@ -397,6 +388,9 @@ int g_lastClientRectx;
 int g_lastClientRecty;
 int g_lastClientRectw;
 int g_lastClientRecth;
+/**
+ * The width of the physical screen in millimeters.
+ */
 double g_display_size_mm;
 std::vector<size_t> g_config_display_size_mm;
 bool g_config_display_size_manual;
@@ -481,7 +475,27 @@ bool g_bLookAhead;
 bool g_bskew_comp;
 bool g_bopengl;
 bool g_bSoftwareGL;
+/**
+ * Controls how the chart panning and zooming smoothing is done during user
+ * interactions.
+ *
+ * When enabled (true):
+ * - Chart panning has inertia, with smooth acceleration and deceleration
+ * - Mouse wheel zooming is smoothly animated between zoom levels
+ * - Chart overscaled rendering is optimized for smooth transitions
+ * - Chart quilting transitions may be smoother
+ *
+ * When disabled (false):
+ * - Chart panning stops immediately when mouse is released
+ * - Mouse wheel zooming jumps directly between zoom levels without animation
+ * - Rendering may be slightly faster but less visually polished
+ *
+ * Changed through the Display > Advanced > "Smooth Panning/Zooming" checkbox.
+ * Saved to config as user preference.
+ */
 bool g_bsmoothpanzoom;
+// toggle for smooth position jumping
+bool g_bSmoothRecenter = true;
 bool g_fog_overzoom;
 double g_overzoom_emphasis_base;
 bool g_oz_vector_scale;
@@ -506,7 +520,6 @@ S57QueryDialog *g_pObjectQueryDialog;
 std::vector<std::string> TideCurrentDataSet;
 wxString g_TCData_Dir;
 
-bool g_boptionsactive;
 options *g_options;
 bool g_bDeferredInitDone;
 int options_lastPage = 0;
@@ -559,14 +572,23 @@ int g_NeedDBUpdate;  // 0 - No update needed, 1 - Update needed because there is
                      // no chart database, inform user, 2 - Start update right
                      // away
 bool g_bPreserveScaleOnX;
+bool g_CanvasHideNotificationIcon;
 
 AboutFrameImpl *g_pAboutDlg;
-about *g_pAboutDlgLegacy;
+About *g_pAboutDlgLegacy;
 
 #if wxUSE_XLOCALE || !wxCHECK_VERSION(3, 0, 0)
 wxLocale *plocale_def_lang = 0;
 #endif
 
+/**
+ * Global locale setting for OpenCPN UI.
+ *
+ * If not set in config (empty string), uses system default locale.
+ * Stores the language/locale name in format "en_US", "fr_FR", etc.
+ * A valid setting triggers loading the corresponding .mo translation files
+ * from the appropriate locale directory.
+ */
 wxString g_locale;
 wxString g_localeOverride;
 bool g_b_assume_azerty;
@@ -625,7 +647,20 @@ wxString g_config_version_string;
 
 wxString g_CmdSoundString;
 
+/**
+ * Flag to control adaptive UI scaling.
+ *
+ * When true, OpenCPN will automatically maximize the application window
+ * if the pixel density suggests a touch-friendly device.
+ *
+ * This helps ensure better usability on mobile and tablet devices by
+ * providing a full-screen interface optimized for touch interaction.
+ *
+ * @note For the most part, the use of this feature is conditionally compiled
+ * for Android builds only.
+ */
 bool g_bresponsive;
+/** Flag to enable or disable mouse rollover effects in the user interface. */
 bool g_bRollover;
 
 bool b_inCompressAllCharts;
@@ -661,7 +696,9 @@ ChartCanvas *g_focusCanvas;
 ChartCanvas *g_overlayCanvas;
 
 bool b_inCloseWindow;
-extern int ShowNavWarning();
+bool g_disable_main_toolbar;
+bool g_btenhertz;
+bool g_declutter_anchorage;
 
 #ifdef LINUX_CRASHRPT
 wxCrashPrint g_crashprint;
@@ -677,12 +714,8 @@ DEFINE_GUID(GARMIN_DETECT_GUID, 0x2c9c45c2L, 0x8e7d, 0x4c08, 0xa1, 0x2d, 0x81,
             0x6b, 0xba, 0xe7, 0x22, 0xc0);
 #endif
 
-#ifdef __MSVC__
-#define _CRTDBG_MAP_ALLOC
-#include <stdlib.h>
-#include <crtdbg.h>
-#define DEBUG_NEW new (_NORMAL_BLOCK, __FILE__, __LINE__)
-#define new DEBUG_NEW
+#ifdef __VISUALC__
+#include <wx/msw/msvcrt.h>
 #endif
 
 #if !defined(NAN)
@@ -699,7 +732,7 @@ void SetSystemColors(ColorScheme cs);
 
 static bool LoadAllPlugIns(bool load_enabled) {
   g_Platform->ShowBusySpinner();
-  bool b = PluginLoader::getInstance()->LoadAllPlugIns(load_enabled);
+  bool b = PluginLoader::GetInstance()->LoadAllPlugIns(load_enabled);
   g_Platform->HideBusySpinner();
   return b;
 }
@@ -986,7 +1019,7 @@ MyApp::MyApp()
       "mesa_glthread", "false",
       1);  // Explicitly disable glthread. This may have some impact on OpenGL
            // performance, but we know it is problematic for us. See #2889
-#endif  // __linux__
+#endif     // __linux__
 }
 
 bool MyApp::OnInit() {
@@ -1111,14 +1144,19 @@ bool MyApp::OnInit() {
                    wxFONTENCODING_SYSTEM);
   temp_font.SetDefaultEncoding(wxFONTENCODING_SYSTEM);
 
+  //  Start the Notification Manager and remove old persisted messages
+  auto &noteman = NotificationManager::GetInstance();
+  noteman.ScrubNotificationDirectory(30);
+
   //      Establish Log File location
-  if (!g_Platform->InitializeLogFile()) return false;
+  if (!g_Platform->InitializeLogFile()) {
+    return false;
+  };
 
 #ifdef __WXMSW__
 
-    //  Un-comment the following to establish a separate console window as a
-    //  target for printf() in Windows
-    //     RedirectIOToConsole();
+  //  Un-comment the following to establish a separate console window as a
+  //  target for printf() in Windows RedirectIOToConsole();
 
 #endif
 
@@ -1194,10 +1232,8 @@ bool MyApp::OnInit() {
   pMessageOnceArray = new wxArrayString;
 
   //      Init the Route Manager
-
   g_pRouteMan =
-      new Routeman(RoutePropDlg::GetDlgCtx(), RoutemanGui::GetDlgCtx(),
-                   NMEALogWindow::GetInstance());
+      new Routeman(RoutePropDlg::GetDlgCtx(), RoutemanGui::GetDlgCtx());
 
   //      Init the Selectable Route Items List
   pSelect = new Select();
@@ -1237,6 +1273,9 @@ bool MyApp::OnInit() {
   //  Routes
   pRouteList = new RouteList;
 
+  //  Initialize the NavObj_db
+  auto &navobj_db = NavObj_dB::GetInstance();
+
   //      (Optionally) Capture the user and file(effective) ids
   //  Some build environments may need root privileges for hardware
   //  port I/O, as in the NMEA data input class.  Set that up here.
@@ -1338,10 +1377,12 @@ bool MyApp::OnInit() {
   }
 
   //  Is this the first run after a clean installation?
-  if (!n_NavMessageShown) g_bFirstRun = true;
+  if (!n_NavMessageShown) {
+    g_bFirstRun = true;
+  }
 
-    //  Now we can set the locale
-    //    using wxWidgets/gettext methodology....
+  //  Now we can set the locale
+  //    using wxWidgets/gettext methodology....
 
 #if wxUSE_XLOCALE || !wxCHECK_VERSION(3, 0, 0)
 
@@ -1404,10 +1445,11 @@ bool MyApp::OnInit() {
   g_Platform->SetUpgradeOptions(vs, g_config_version_string);
 
   //  log deferred log restart message, if it exists.
-  if (!g_Platform->GetLargeLogMessage().IsEmpty())
+  if (!g_Platform->GetLargeLogMessage().IsEmpty()) {
     wxLogMessage(g_Platform->GetLargeLogMessage());
+  }
 
-    //  Validate OpenGL functionality, if selected
+  // Validate OpenGL functionality, if selected
 #ifndef ocpnUSE_GL
   g_bdisable_opengl = true;
   ;
@@ -1416,17 +1458,19 @@ bool MyApp::OnInit() {
   if (g_bdisable_opengl) g_bopengl = false;
 
 #if defined(__linux__) && !defined(__ANDROID__)
-  if (g_bSoftwareGL) setenv("LIBGL_ALWAYS_SOFTWARE", "1", 1);
+  if (g_bSoftwareGL) {
+    setenv("LIBGL_ALWAYS_SOFTWARE", "1", 1);
+  }
 #endif
 
-    // FIXMW (dave) move to frame
-    // g_bTransparentToolbarInOpenGLOK = isTransparentToolbarInOpenGLOK();
+  // FIXMW (dave) move to frame
+  // g_bTransparentToolbarInOpenGLOK = isTransparentToolbarInOpenGLOK();
 
-    // On Windows platforms, establish a default cache managment policy
-    // as allowing OpenCPN a percentage of available physical memory,
-    // not to exceed 1 GB
-    // Note that this logic implies that Windows platforms always use
-    // the memCacheLimit policy, and never use the fallback nCacheLimit policy
+  // On Windows platforms, establish a default cache managment policy
+  // as allowing OpenCPN a percentage of available physical memory,
+  // not to exceed 1 GB
+  // Note that this logic implies that Windows platforms always use
+  // the memCacheLimit policy, and never use the fallback nCacheLimit policy
 #ifdef __WXMSW__
   if (0 == g_memCacheLimit) g_memCacheLimit = (int)(g_mem_total * 0.5);
   g_memCacheLimit =
@@ -1511,7 +1555,7 @@ bool MyApp::OnInit() {
   auto style = g_StyleManager->GetCurrentStyle();
   auto bitmap = new wxBitmap(style->GetIcon("default_pi", 32, 32));
   if (bitmap->IsOk())
-    PluginLoader::getInstance()->SetPluginDefaultIcon(bitmap);
+    PluginLoader::GetInstance()->SetPluginDefaultIcon(bitmap);
   else
     wxLogWarning("Cannot initiate plugin default jigsaw icon.");
 
@@ -1607,7 +1651,8 @@ bool MyApp::OnInit() {
   wxLogMessage(fmsg);
 
   gFrame = new MyFrame(NULL, myframe_window_title, position, new_frame_size,
-                       app_style);  // Gunther
+                       app_style);
+  wxTheApp->SetTopWindow(gFrame);
 
   //  Do those platform specific initialization things that need gFrame
   g_Platform->Initialize_3();
@@ -1845,6 +1890,9 @@ bool MyApp::OnInit() {
   //      Start up the ViewPort Rotation angle Averaging Timer....
   gFrame->FrameCOGTimer.Start(2000, wxTIMER_CONTINUOUS);
 
+  //      Start up the Ten Hz timer....
+  gFrame->FrameTenHzTimer.Start(100, wxTIMER_CONTINUOUS);
+
   //    wxLogMessage( wxString::Format(_T("OpenCPN Initialized in %ld ms."),
   //    init_sw.Time() ) );
 
@@ -1867,7 +1915,7 @@ bool MyApp::OnInit() {
     // qDebug() << "Showing NavWarning";
     wxMilliSleep(500);
 
-    if (wxID_CANCEL == ShowNavWarning()) {
+    if (!ShowNavWarning()) {
       qDebug() << "Closing due to NavWarning Cancel";
       gFrame->Close();
       androidTerminate();
@@ -1885,7 +1933,7 @@ bool MyApp::OnInit() {
   //  or if the version string has changed at all
   //  We defer until here to allow for localization of the message
   if (!n_NavMessageShown || (vs != g_config_version_string)) {
-    if (wxID_CANCEL == ShowNavWarning()) return false;
+    if (!ShowNavWarning()) return false;
     n_NavMessageShown = 1;
     pConfig->Flush();
   }
@@ -1911,8 +1959,7 @@ bool MyApp::OnInit() {
 
   g_pauimgr->Update();
 
-  for (size_t i = 0; i < TheConnectionParams()->Count(); i++) {
-    ConnectionParams *cp = TheConnectionParams()->Item(i);
+  for (auto *cp : TheConnectionParams()) {
     if (cp->bEnabled) {
       if (cp->GetDSPort().Contains("Serial")) {
         std::string port(cp->Port.ToStdString());
@@ -2042,12 +2089,14 @@ int MyApp::OnExit() {
 #ifdef __WXMSW__
 #ifdef USE_GLU_TESS
 #ifdef USE_GLU_DLL
-  if (s_glu_dll_ready) FreeLibrary(s_hGLU_DLL);  // free the glu32.dll
+  if (s_glu_dll_ready) {
+    FreeLibrary(s_hGLU_DLL);
+  }  // free the glu32.dll
 #endif
 #endif
 #endif
 
-    //      Restore any changed system colors
+  // Restore any changed system colors
 
 #ifdef __WXMSW__
   void RestoreSystemColors(void);
