@@ -63,6 +63,8 @@ enum {
   ID_TBSTAT,
   ID_TBSTATBOX,
   ID_MASTERTOGGLE,
+  ID_SETTINGS_NEW,
+  ID_SETTINGS_DELETE,
 
   ID_PLUGIN_BASE  // This MUST be the last item in the enum
 };
@@ -117,6 +119,8 @@ enum {
   ID_MENU_SHOW_CURRENTS,
 
   ID_MENU_TOOL_MEASURE,
+  ID_MENU_TOOL_NMEA_DBG_LOG,
+  ID_MENU_TOOL_IO_MONITOR,
   ID_MENU_ROUTE_MANAGER,
   ID_MENU_ROUTE_NEW,
   ID_MENU_MARK_BOAT,
@@ -164,7 +168,8 @@ enum {
   RESIZE_TIMER,
   TOOLBAR_ANIMATE_TIMER,
   RECAPTURE_TIMER,
-  WATCHDOG_TIMER
+  WATCHDOG_TIMER,
+  FRANE_TENHZ_TIMER
 
 };
 
