@@ -125,10 +125,10 @@ EVT_TIMER(TIMER_GARMIN1, GarminProtocolHandler::OnTimerGarmin1)
 END_EVENT_TABLE()
 
 GarminProtocolHandler::GarminProtocolHandler(wxString port,
-                                             wxEvtHandler *MessageTarget,
+                                             SendMsgFunc send_msg_func,
                                              bool bsel_usb) {
   // m_pparent = parent;
-  m_pMainEventHandler = MessageTarget;
+  m_send_msg_func = send_msg_func;
   m_garmin_serial_thread = NULL;
   m_garmin_usb_thread = NULL;
   m_bOK = false;
@@ -166,7 +166,7 @@ GarminProtocolHandler::GarminProtocolHandler(wxString port,
 
     // Start handler thread
     m_garmin_serial_thread =
-        new GARMIN_Serial_Thread(this, m_pMainEventHandler, m_port);
+        new GARMIN_Serial_Thread(this, m_send_msg_func, m_port);
 
     m_Thread_run_flag = 1;
     m_garmin_serial_thread->Run();
@@ -258,7 +258,7 @@ void GarminProtocolHandler::OnTimerGarmin1(wxTimerEvent &event) {
 
         //    Start the pump
         m_garmin_usb_thread = new GARMIN_USB_Thread(
-            this, m_pMainEventHandler, (wxIntPtr)m_usb_handle, m_max_tx_size);
+            this, m_send_msg_func, (wxIntPtr)m_usb_handle, m_max_tx_size);
         m_Thread_run_flag = 1;
         m_garmin_usb_thread->Run();
       }
@@ -693,10 +693,10 @@ D800_Pvt_Data_Type_Aligned mypvt;
 //
 //-------------------------------------------------------------------------------------------------------------
 GARMIN_Serial_Thread::GARMIN_Serial_Thread(GarminProtocolHandler *parent,
-                                           wxEvtHandler *MessageTarget,
+                                           SendMsgFunc send_msg_func,
                                            wxString port) {
   m_parent = parent;  // This thread's immediate "parent"
-  m_pMessageTarget = MessageTarget;
+  m_send_msg_func = send_msg_func;
   m_port = port;
 
   Create();
@@ -781,19 +781,10 @@ void *GARMIN_Serial_Thread::Entry() {
           oNMEA0183.Rmc.IsDataValid = NTrue;
 
           oNMEA0183.Rmc.Write(snt);
-          wxString message = snt.Sentence;
 
-          //    Copy the message into a vector for tranmittal upstream
-          auto buffer = std::make_shared<std::vector<unsigned char>>();
-          std::vector<unsigned char> *vec = buffer.get();
-          for (unsigned int i = 0; i < message.Length(); i++) {
-            vec->push_back(message[i]);
-          }
-          if (m_pMessageTarget) {
-            CommDriverN0183SerialEvent Nevent(wxEVT_COMMDRIVER_N0183_SERIAL, 0);
-            Nevent.SetPayload(buffer);
-            m_pMessageTarget->AddPendingEvent(Nevent);
-          }
+          // Send message...
+          auto msg = snt.Sentence.ToStdString();
+          m_send_msg_func(std::vector<unsigned char>(msg.begin(), msg.end()));
 
           last_rx_time = wxDateTime::Now();
         }
@@ -837,11 +828,11 @@ thread_exit:
 //-------------------------------------------------------------------------------------------------------------
 #if 1
 GARMIN_USB_Thread::GARMIN_USB_Thread(GarminProtocolHandler *parent,
-                                     wxEvtHandler *MessageTarget,
+                                     SendMsgFunc send_msg_func,
                                      unsigned int device_handle,
                                      size_t max_tx_size) {
   m_parent = parent;  // This thread's immediate "parent"
-  m_pMessageTarget = MessageTarget;
+  m_send_msg_func = send_msg_func;
   m_max_tx_size = max_tx_size;
 
 #ifdef __WXMSW__
@@ -891,19 +882,10 @@ void *GARMIN_USB_Thread::Entry() {
       oNMEA0183.Gsv.SatsInView = m_nSats;
 
       oNMEA0183.Gsv.Write(snt);
-      wxString message = snt.Sentence;
+      wxString msg = snt.Sentence;
 
-      //    Copy the message into a vector for tranmittal upstream
-      auto buffer = std::make_shared<std::vector<unsigned char>>();
-      std::vector<unsigned char> *vec = buffer.get();
-      for (unsigned int i = 0; i < message.Length(); i++) {
-        vec->push_back(message[i]);
-      }
-      if (m_pMessageTarget) {
-        CommDriverN0183SerialEvent Nevent(wxEVT_COMMDRIVER_N0183_SERIAL, 0);
-        Nevent.SetPayload(buffer);
-        m_pMessageTarget->AddPendingEvent(Nevent);
-      }
+      // and send message.
+      m_send_msg_func(std::vector<unsigned char>(msg.begin(), msg.end()));
     }
 
     if (iresp.gusb_pkt.pkt_id[0] == GUSB_RESPONSE_PVT)  // PVT Data Record
@@ -945,19 +927,8 @@ void *GARMIN_USB_Thread::Entry() {
         oNMEA0183.Rmc.IsDataValid = NTrue;
 
         oNMEA0183.Rmc.Write(snt);
-        wxString message = snt.Sentence;
-
-        //    Copy the message into a vector for tranmittal upstream
-        auto buffer = std::make_shared<std::vector<unsigned char>>();
-        std::vector<unsigned char> *vec = buffer.get();
-        for (unsigned int i = 0; i < message.Length(); i++) {
-          vec->push_back(message[i]);
-        }
-        if (m_pMessageTarget) {
-          CommDriverN0183SerialEvent Nevent(wxEVT_COMMDRIVER_N0183_SERIAL, 0);
-          Nevent.SetPayload(buffer);
-          m_pMessageTarget->AddPendingEvent(Nevent);
-        }
+        wxString msg = snt.Sentence;
+        m_send_msg_func(std::vector<unsigned char>(msg.begin(), msg.end()));
       }
     }
   }
